@@ -1,9 +1,11 @@
 import cStringIO
+from netlib import odict
 from libmproxy import filt, flow
 from libmproxy.protocol import http
 from libmproxy.protocol.primitives import Error
 import tutils
 
+
 class TestParsing:
     def _dump(self, x):
         c = cStringIO.StringIO()
@@ -74,7 +76,7 @@ class TestParsing:
 
 class TestMatching:
     def req(self):
-        headers = flow.ODictCaseless()
+        headers = odict.ODictCaseless()
         headers["header"] = ["qvalue"]
         req = http.HTTPRequest(
             "absolute",
@@ -96,9 +98,17 @@ class TestMatching:
     def resp(self):
         f = self.req()
 
-        headers = flow.ODictCaseless()
+        headers = odict.ODictCaseless()
         headers["header_response"] = ["svalue"]
-        f.response = http.HTTPResponse((1, 1), 200, "OK", headers, "content_response", None, None)
+        f.response = http.HTTPResponse(
+            (1,
+             1),
+            200,
+            "OK",
+            headers,
+            "content_response",
+            None,
+            None)
 
         return f
 
@@ -231,6 +241,23 @@ class TestMatching:
         assert self.q("~c 200", s)
         assert not self.q("~c 201", s)
 
+    def test_src(self):
+        q = self.req()
+        assert self.q("~src address", q)
+        assert not self.q("~src foobar", q)
+        assert self.q("~src :22", q)
+        assert not self.q("~src :99", q)
+        assert self.q("~src address:22", q)
+
+    def test_dst(self):
+        q = self.req()
+        q.server_conn = tutils.tserver_conn()
+        assert self.q("~dst address", q)
+        assert not self.q("~dst foobar", q)
+        assert self.q("~dst :22", q)
+        assert not self.q("~dst :99", q)
+        assert self.q("~dst address:22", q)
+
     def test_and(self):
         s = self.resp()
         assert self.q("~c 200 & ~h head", s)
@@ -253,4 +280,3 @@ class TestMatching:
         assert self.q("! ~c 201", s)
         assert self.q("!~c 201 !~c 202", s)
         assert not self.q("!~c 201 !~c 200", s)
-
